__version__ = "0.0.3"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.0.3)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'ea94f7acb13b56f75661a87f74457d8ec4924b66'
__git_head_subject__ = 'HOSTDEV-1978 LogbookPlugin: does not enable logbook compatibility if gevent not present'
__git_head_message__ = ''
__git_dirty_diff__ = ''